@@ -14,6 +14,7 @@ of having everything pre-defined.
 
 import sys
 import os
+import functools
 
 from plib.stdlib.builtins import first
 
@@ -27,7 +28,7 @@ from plib.ui import custom
 custom.add_signals(
     SIGNAL_CUSTOM1=(('SIGNAL_FINISHED', 50), 'custom1'),
     SIGNAL_CUSTOM2=(('SIGNAL_FINISHED', 51), 'custom2'),
-    SIGNAL_EMITTED=(('SIGNAL_FINISHED', 60), 'emitted'),
+    SIGNAL_EMITTED=(('SIGNAL_FINISHED', 60), 'emitted', int),
 )
 
 custom.add_widget_prefixes(
@@ -38,65 +39,40 @@ custom.add_widget_prefixes(
 
 from plib.ui import __version__
 from plib.ui.defs import *
-from plib.ui.app import PApplication
+from plib.ui.app import PApplication, PTextLabel, PMouseMixin
 from plib.ui.dialogs import StringSelectDialog
-from plib.ui.imp import get_toolkit_object
 from plib.ui.manager import PWidgetManager
 from plib.ui.output import PTextOutput
-from plib.ui.toolkits import get_toolkit
 from plib.ui.widgets import *
 
 
-# TODO: is there a way to add an API to custom to avoid toolkit-specific code here?
-
-class MouseBase:
-    
-    mouse_pressed = False
+class MouseMixin(PMouseMixin):
     
     @property
     def event_signal(self):
         return self.signals[0]
-
-
-toolkit = get_toolkit()
-
-if toolkit in ('pyside2', 'qt5'):
     
-    class MouseMixin(MouseBase):
-        
-        def mousePressEvent(self, event):
-            self.mouse_pressed = True
-        
-        def mouseReleaseEvent(self, event):
-            if self.mouse_pressed:
-                self.do_notify(self.event_signal)
-            self.mouse_pressed = False
-
-
-elif toolkit == 'wx':
+    msg_signames = (
+        'SIGNAL_LEFTCLICK',
+        'SIGNAL_RIGHTCLICK',
+        'SIGNAL_MIDDLECLICK',
+        'SIGNAL_LEFTDBLCLICK',
+        'SIGNAL_RIGHTDBLCLICK',
+        'SIGNAL_MIDDLEDBLCLICK',
+    )
     
-    class MouseMixin(MouseBase):
-        
-        def setup_signals(self):
-            import wx
-            self.wx_register_event(wx.EVT_LEFT_DOWN, self.OnLeftDown)
-            self.wx_register_event(wx.EVT_LEFT_UP, self.OnLeftUp)
-        
-        def OnLeftDown(self, event):
-            self.mouse_pressed = True
-        
-        def OnLeftUp(self, event):
-            if self.mouse_pressed:
-                self.forward_event(self.event_signal, event)
-            self.mouse_pressed = False
-
-
-else:
-    # This should never happen
-    raise RuntimeError("Unknown UI toolkit {}".format(toolkit))
-
-
-PTextLabel = get_toolkit_object('label', 'PTextLabel')
+    def setup_signals(self):
+        super(MouseMixin, self).setup_signals()
+        for signame in self.msg_signames:
+            sig = getattr(sys.modules[__name__], signame)
+            self.setup_notify(sig, functools.partial(self.on_mouse_signal, signame))
+        self.setup_notify(SIGNAL_LEFTCLICK, self.on_left_click)
+    
+    def on_mouse_signal(self, signame):
+        self.manager.app.output_message("From Page {}: {}".format(self.__class__.__name__, signame))
+    
+    def on_left_click(self):
+        self.do_notify(self.event_signal, *self.event_args)
 
 
 class ClickableLabel(MouseMixin, PTextLabel):
@@ -104,6 +80,8 @@ class ClickableLabel(MouseMixin, PTextLabel):
     signals = (
         SIGNAL_CLICKED,
     )
+    
+    event_args = ()
 
 
 class CustomWidget(MouseMixin, PTextLabel):
@@ -111,6 +89,14 @@ class CustomWidget(MouseMixin, PTextLabel):
     signals = (
         SIGNAL_EMITTED,
     )
+    
+    @property
+    def event_args(self,
+                   sto=[0]):
+        
+        num = sto[0]
+        sto[0] += 1
+        return (num,)
 
 
 def clickable_label(name, caption, geometry=None, font=None):
@@ -203,8 +189,8 @@ class Page(PWidgetManager):
         print("custom2_clicked")
         self.app.do_notify(SIGNAL_CUSTOM2)
     
-    def on_custom3_emitted(self):
-        self.app.output_message("From Page CustomWidget: SIGNAL_EMITTED")
+    def on_custom3_emitted(self, num):
+        self.app.output_message("From Page CustomWidget: SIGNAL_EMITTED {}".format(num))
 
 
 PAGE_TABS = 0
