@@ -97,12 +97,19 @@ event_map = {
     SIGNAL_BUTTONSELECTED: ("sig_buttonSelected", (int,)),
     SIGNAL_CELLCHANGED: ("sig_cellChanged", (int, int)),
     SIGNAL_TEXTSTATECHANGED: ("sig_textStateChanged", ()),  # using selectionChanged above does not work
-    SIGNAL_FOCUS_IN: ("sig_focusInEvent", ()),
-    SIGNAL_FOCUS_OUT: ("sig_focusOutEvent", ()),
     SIGNAL_QUERYCLOSE: ("sig_canClose", ()),
     SIGNAL_CLOSING: ("sig_closeEvent", ()),
     SIGNAL_SHOWN: ("sig_showEvent", ()),
     SIGNAL_HIDDEN: ("sig_hideEvent", ()),
+    
+    SIGNAL_FOCUS_IN: ("sig_focusInEvent", ()),
+    SIGNAL_FOCUS_OUT: ("sig_focusOutEvent", ()),
+    SIGNAL_LEFTCLICK: ("sig_leftClickEvent", ()),
+    SIGNAL_RIGHTCLICK: ("sig_rightClickEvent", ()),
+    SIGNAL_MIDDLECLICK: ("sig_middleClickEvent", ()),
+    SIGNAL_LEFTDBLCLICK: ("sig_leftDblClickEvent", ()),
+    SIGNAL_RIGHTDBLCLICK: ("sig_rightDblClickEvent", ()),
+    SIGNAL_MIDDLEDBLCLICK: ("sig_middleDblClickEvent", ()),
 }
 
 
@@ -233,8 +240,6 @@ class PQtWidget(PQtSignal, metaclass=PQtWidgetMeta):
     fn_enable_get = 'isEnabled'
     fn_enable_set = 'setEnabled'
     
-    event_signals = (SIGNAL_FOCUS_IN, SIGNAL_FOCUS_OUT)
-    
     fix_width_on_resize = False
     fix_height_on_resize = False
     
@@ -333,14 +338,6 @@ class PQtWidget(PQtSignal, metaclass=PQtWidgetMeta):
     
     def set_focus(self):
         self.setFocus()
-    
-    def focusInEvent(self, event):
-        super(PQtWidget, self).focusInEvent(event)
-        self.do_notify(SIGNAL_FOCUS_IN)
-    
-    def focusOutEvent(self, event):
-        super(PQtWidget, self).focusOutEvent(event)
-        self.do_notify(SIGNAL_FOCUS_OUT)
 
 
 class PQtSequenceWidget(PQtWidget, metaclass=PQtSequenceWidgetMeta):
